#include "utility.lua"
#include "debug.lua"
#include "umf.lua"

Projectiles = {}

function initMis()
	initProjectiles()
	count = 1
end

function updateMis()
	emitterTr = GetLightTransform(FindLight("emitter"))
	bklsTr = GetLightTransform(FindLight("backlash"))
	targets = radarStat.target
	
	
	if InputPressed("-") then
		count = count - 1
	elseif InputPressed("+") then
		count = count + 1
	end
	
	if count < 1 then
		count = #targets
	elseif count > #targets then
		count = 1
	end
	
	for i=1, #targets do
		if i == count then
			targets[i].locked = true
		else
			targets[i].locked = false
		end
	end
	
	manageActiveProjectiles()
	if InputPressed("rmb") then
		createProjectile(
			emitterTr,
			Projectiles,
			ProjectilePresets.missiles["9M330"],
			nil,
			GetBodyShapes(GetVehicleBody(targets[count].handle))[1]
		)
	end
end

function initProjectiles()

    ProjectilePresets = {
	
        missiles = {

            ["9M330"] = {
				
                isActive = true, -- Active when firing, inactive after hit.
                hit = false,
                hitInitial = false,
				backlashed = false,	--whether backlash effect has finished
                lifeLength = 10, --Seconds

                category = 'missile',
				launchType = 'cold',

                speed = 2,
				pushed = false,
				--params below are exclusively for cold launch
				ejected = false,	--eject flag
				ejectSpd = 1,		--initial ejection speed
				backlashed = false,
				deflected = false,
				deflectPropel = false,
				gravityCoef = 0.9,	--gravity coefficient
				ignitionSpd = 0.2,	--threshold speed to start rocket
				deflectOnSpdThr = 0.5,		--threshold speed to start deflection rocket
				deflectOffSpdThr = 0.3,
				afterburn = 0.05,
				--end
                spread = 0,
                drop = 0,
                dropIncrement = 0,
                explosionSize = 3.5,
                rcRad = 0.5,
                force = 10,
                penetrate = false,
                holeSize = 0,
				
				physics = {
					localVelVecs = {},
					localCompVelVec = Vec(0,0,0),
					localFwdVec = Vec(0,0,-1),
					gravityVec = Vec(0,-1,0),
					gravityCoef = 0.98,
					shellMass = 1,
				},

                effects = {
                    particle = 'smoke',
                    color = Vec(1,0.5,0.3),
                    sprite = 'MOD/HomingMissile/res/sprite/9M330.png',
                    sprite_dimensions = {5, 1},
                    sprite_facePlayer = false,
					propelSound = LoadLoop("MOD/HomingMissile/res/sound/rocketFlightLoop0.ogg")
                },

                sounds = {
                    -- hit = Sounds.weap_secondary.hit,
                    hit_vol = 5,
                },

                homing = {
                    force = 0.05,	--maneuverability, higher = better
                    gain = 0,
                    max = 10,
                    targetShape = nil,
                    targetPos = Vec(),
                    targetPosRadius = 0,
                }
            }

        },

    }

end

function createProjectile(transform, projectiles, projPreset, ignoreBodies, homingShape) --- Instantiates a proj and adds it to the projectiles table.
	
    local proj = DeepCopy(projPreset)
    proj.ignoreBodies = DeepCopy(ignoreBodies)

    proj.transform = transform
    local trDir = QuatToDir(proj.transform.rot)
    trDir = VecNormalize(VecAdd(trDir, VecRdm(proj.spread)))
    proj.transform.rot = DirToQuat(trDir)
	
	--initialize target position
    if proj.homing.max > 0 and homingShape ~= nil then
        proj.homing.targetShape = homingShape
        proj.homing.targetPos = GetShapeLocalTransform(homingShape).pos
    end

    table.insert(projectiles, proj)
end

function manageActiveProjectiles()

    local projectilesToRemove = {} -- projectiles iterations.
    for i, proj in ipairs(Projectiles) do

        if proj.isActive and proj.hit == false then
			if proj.launchType == "cold" and (proj.ejected == false or proj.deflected == false) then
				ejectProjectile(proj,emitterTr,bklsTr)
			else
				propelProjectile(proj)
			end

        elseif proj.isActive == false or proj.hit then -- if proj is inactive.

            table.insert(projectilesToRemove, i)

        end

    end

    for i = 1, #projectilesToRemove do -- remove proj from active projs after projectiles iterations.
        table.remove(Projectiles, projectilesToRemove[i]) -- remove active projs
    end

end

function ejectProjectile(proj, emitterTr, bklsTr)
	
    --+ Eject proj.
	
	--gravity effect
	
		--get current fwd dir for later calculation of propelling force
		local globalFwdPoint = TransformToParentPoint(proj.transform, proj.physics.localFwdVec)
		local localFwdVec = VecNormalize(QuatRotateVec(QuatLookAt(proj.transform.pos,globalFwdPoint), proj.physics.localFwdVec))
		--end
		
		--numerate incurred forces
			--gravity
		local localGravityVec = VecScale(proj.physics.gravityVec, (proj.physics.gravityCoef * proj.physics.shellMass) * GetTimeStep())
		proj.physics.localCompVelVec = VecAdd(proj.physics.localCompVelVec, localGravityVec)
			--v = at
		
			--ejection force (only imposed at beginning)
		local localPropelVec = Vec(0,0,0)
		if not proj.pushed then
			localPropelVec = VecScale(localFwdVec, proj.ejectSpd)
			proj.pushed = true
		else
			localPropelVec = VecScale(localFwdVec, 0)
		end
		proj.physics.localCompVelVec = VecAdd(proj.physics.localCompVelVec, localPropelVec)
		--end
		
		--acc magnitude
		local accMag = proj.physics.localCompVelVec[2]	--judge when to start rocket according to vertical vel
		--end
		
		--move local accVec to projectile position
		local adjustedPosVec = VecAdd(proj.physics.localCompVelVec, proj.transform.pos)
		--end
	
	--end
	
	proj.transform.pos = adjustedPosVec
	
	--+ Ignore bodies
    if proj.ignoreBodies ~= nil then
        for i = 1, #proj.ignoreBodies do
            QueryRejectBody(proj.ignoreBodies[i])
        end
    end

    --+ Raycast
    local rcHit, hitPos, hitShape = RaycastFromTransform(proj.transform, proj.speed, proj.rcRad, proj.ignoreBodies, nil, true)
    if rcHit and not proj.hitInitial then

        proj.hitInitial = true
        proj.hit = true

        --+ Hit Action
        -- ApplyBodyImpulse(GetShapeBody(hitShape), hitPos, VecScale(QuatToDir(proj.transform.rot), proj.force))

        if proj.explosionSize > 0 then
            Explosion(hitPos, proj.explosionSize)
        end

        --+ Sounds
        -- local index = proj.sounds.hit[math.random(1, #proj.sounds.hit)]
        -- PlayRandomSound(proj.sounds.hit, proj.transform.pos, 2, index)
        -- PlayRandomSound(proj.sounds.hit, GetCameraTransform().pos, 0.2 + math.random()/10, index)

    end

    -- if proj.penetrate then

    --     proj.hit = false

    --     if VecDist(robot.transform.pos, proj.transform.pos) > proj.holeSize * 2 then
    --         MakeHole(proj.transform.pos, proj.holeSize,proj.holeSize,proj.holeSize,proj.holeSize)
    --     end

    -- end

    --+ Proj hit water.
    if IsPointInWater(proj.transform.pos) then
        proj.hit = true
        SpawnParticle("water", proj.transform.pos, Vec(0,0,0))
    end

    --+ Proj life.
    if proj.hit then
        proj.isActive = false
    else
        proj.isActive = true
    end
	
	--+ Proj homing(deflecting).
    if proj.homing.max > 0 and proj.homing.targetShape ~= nil and proj.ejected then
        local b = GetShapeBody(proj.homing.targetShape)
        proj.homing.targetPos = AabbGetBodyCenterPos(b)
        -- proj.homing.targetPos = GetBodyTransform(b).pos

        dbl(proj.transform.pos, proj.homing.targetPos, 1,0,1, 1)
        dbray(proj.transform, 30, 0,1,1, 1)

        proj.transform.rot = MakeQuaternion(proj.transform.rot)
        proj.transform.rot = proj.transform.rot:Approach(QuatLookAt(proj.transform.pos, proj.homing.targetPos), proj.homing.force*1.5) -- Rotate towards homing target.

        if proj.homing.force < proj.homing.max then -- Increment homing strength.
            proj.homing.force = proj.homing.force + proj.homing.gain
        end

    end

    --+ Draw sprite
    if proj.effects.sprite_facePlayer then
        DrawSprite(LoadSprite(proj.effects.sprite), Transform(proj.transform.pos, QuatLookAt(proj.transform.pos, GetCameraTransform().pos)), proj.effects.sprite_dimensions[1], proj.effects.sprite_dimensions[2], 1, 1, 1, 1, true)
    else
        DrawSprite(LoadSprite(proj.effects.sprite), Transform(proj.transform.pos, QuatRotateQuat(proj.transform.rot, QuatEuler(90,-90,0))), proj.effects.sprite_dimensions[1], proj.effects.sprite_dimensions[2], 1, 1, 1, 1, true)
        DrawSprite(LoadSprite(proj.effects.sprite), Transform(proj.transform.pos, QuatRotateQuat(proj.transform.rot, QuatEuler(0,-90,0))), proj.effects.sprite_dimensions[1], proj.effects.sprite_dimensions[2], 1, 1, 1, 1, true)
    end
	
	 --+ Particles
    if proj.category == 'missile' and proj.deflectPropel then
        SpawnParticle("fire", TransformToParentPoint(proj.transform, Vec(0,0,-proj.effects.sprite_dimensions[1]/2)), Vec(0,0,0), 1.5, 0.2)
        SpawnParticle("smoke", TransformToParentPoint(proj.transform, Vec(0,0,-proj.effects.sprite_dimensions[1]/2)), Vec(0,0,0), 2.8, 4)
		
		local c = proj.effects.color
		PointLight(TransformToParentPoint(proj.transform, Vec(0,0,-proj.effects.sprite_dimensions[1]/2)), c[1], c[2], c[3], 10)
	end
	
	bklsTr = Transform(bklsTr.pos, QuatLookAt(bklsTr.pos, emitterTr.pos))
	if not proj.backlashed then
		SpawnParticle("fire", bklsTr.pos, TransformToLocalVec(bklsTr,Vec(0,1,0)), 1.5, 0.2)
        SpawnParticle("smoke", bklsTr.pos, TransformToLocalVec(bklsTr,Vec(0,1,0)), 2.8, 4)
		
		local c = proj.effects.color
		PointLight(bklsTr.pos, c[1], c[2], c[3], 50)
		
		proj.backlashed = true
	end
	
	
	if accMag <= proj.deflectOnSpdThr then	--ejected complete, start deflect rocket
		proj.ejected = true
		proj.deflectPropel = true
	end
	
	if accMag <= proj.deflectOffSpdThr then	--deflect complete, turn off deflect rocket
		proj.deflectPropel = false
	end
	
	if accMag <= proj.ignitionSpd then	--reached ignition speed, start propelling
		proj.deflected = true
	end
end

function propelProjectile(proj)
	
    --+ Move proj forward.
	proj.ignitionSpd = proj.ignitionSpd + proj.afterburn
	if proj.ignitionSpd >= proj.speed then
		proj.ignitionSpd = proj.speed
	end
	
	
	--gravity effect
	
		--get current fwd dir for later calculation of propelling force
		local globalFwdPoint = TransformToParentPoint(proj.transform, proj.physics.localFwdVec)
		local localFwdVec = VecNormalize(QuatRotateVec(QuatLookAt(proj.transform.pos,globalFwdPoint), proj.physics.localFwdVec))
		--end
		
		--numerate incurred forces
			--gravity
		local localGravityVec = VecScale(proj.physics.gravityVec, (proj.physics.gravityCoef * proj.physics.shellMass) * GetTimeStep())
		proj.physics.localCompVelVec = VecAdd(proj.physics.localCompVelVec, localGravityVec)
			--v = at
		
		local localPropelVec = Vec(0,0,0)

		localPropelVec = VecScale(localFwdVec, proj.ignitionSpd)
		proj.physics.localCompVelVec = VecAdd(proj.physics.localCompVelVec, localPropelVec)
		--end
		
		--acc magnitude
		local accMag = VecLength(proj.physics.localCompVelVec)
		--end
		
		--move local accVec to projectile position
		local adjustedPosVec = VecAdd(proj.physics.localCompVelVec, proj.transform.pos)
		--end
	
	--end
	
	proj.transform.pos = adjustedPosVec
	
	proj.physics.localCompVelVec = Vec(0,0,0)	--clear all vels to prevent rocket from unconstrained acceleration
	

    proj.lifeLength = proj.lifeLength - GetTimeStep()
    if proj.lifeLength <= 0 then
        proj.hit = true
    end

    --+ Ignore bodies
    if proj.ignoreBodies ~= nil then
        for i = 1, #proj.ignoreBodies do
            QueryRejectBody(proj.ignoreBodies[i])
        end
    end

    --+ Raycast
    local rcHit, hitPos, hitShape = RaycastFromTransform(proj.transform, proj.speed, proj.rcRad, proj.ignoreBodies, nil, true)
    if rcHit and not proj.hitInitial then

        proj.hitInitial = true
        proj.hit = true

        --+ Hit Action
        -- ApplyBodyImpulse(GetShapeBody(hitShape), hitPos, VecScale(QuatToDir(proj.transform.rot), proj.force))

        if proj.explosionSize > 0 then
            Explosion(hitPos, proj.explosionSize)
        end

        --+ Sounds
        -- local index = proj.sounds.hit[math.random(1, #proj.sounds.hit)]
        -- PlayRandomSound(proj.sounds.hit, proj.transform.pos, 2, index)
        -- PlayRandomSound(proj.sounds.hit, GetCameraTransform().pos, 0.2 + math.random()/10, index)

    end

    -- if proj.penetrate then

    --     proj.hit = false

    --     if VecDist(robot.transform.pos, proj.transform.pos) > proj.holeSize * 2 then
    --         MakeHole(proj.transform.pos, proj.holeSize,proj.holeSize,proj.holeSize,proj.holeSize)
    --     end

    -- end

    --+ Proj hit water.
    if IsPointInWater(proj.transform.pos) then
        proj.hit = true
        SpawnParticle("water", proj.transform.pos, Vec(0,0,0))
    end

    --+ Proj life.
    if proj.hit then
        proj.isActive = false
    else
        proj.isActive = true
    end


    --+ Proj homing.
    if proj.homing.max > 0 and proj.homing.targetShape ~= nil then

        local b = GetShapeBody(proj.homing.targetShape)
        proj.homing.targetPos = AabbGetBodyCenterPos(b)
        -- proj.homing.targetPos = GetBodyTransform(b).pos

        dbl(proj.transform.pos, proj.homing.targetPos, 1,0,1, 1)
        dbray(proj.transform, 30, 0,1,1, 1)

        proj.transform.rot = MakeQuaternion(proj.transform.rot)
        proj.transform.rot = proj.transform.rot:Approach(QuatLookAt(proj.transform.pos, proj.homing.targetPos), proj.homing.force) -- Rotate towards homing target.

        if proj.homing.force < proj.homing.max then -- Increment homing strength.
            proj.homing.force = proj.homing.force + proj.homing.gain
        end

    end


    --+ Draw sprite
    if proj.effects.sprite_facePlayer then
        DrawSprite(LoadSprite(proj.effects.sprite), Transform(proj.transform.pos, QuatLookAt(proj.transform.pos, GetCameraTransform().pos)), proj.effects.sprite_dimensions[1], proj.effects.sprite_dimensions[2], 1, 1, 1, 1, true)
    else
        DrawSprite(LoadSprite(proj.effects.sprite), Transform(proj.transform.pos, QuatRotateQuat(proj.transform.rot, QuatEuler(90,-90,0))), proj.effects.sprite_dimensions[1], proj.effects.sprite_dimensions[2], 1, 1, 1, 1, true)
        DrawSprite(LoadSprite(proj.effects.sprite), Transform(proj.transform.pos, QuatRotateQuat(proj.transform.rot, QuatEuler(0,-90,0))), proj.effects.sprite_dimensions[1], proj.effects.sprite_dimensions[2], 1, 1, 1, 1, true)
    end


    --+ Particles
    if proj.category == 'missile' then
        SpawnParticle("fire", TransformToParentPoint(proj.transform, Vec(0,0,proj.effects.sprite_dimensions[1]/2)), Vec(0,0,0), 1.5, 0.2)
        SpawnParticle("smoke", TransformToParentPoint(proj.transform, Vec(0,0,proj.effects.sprite_dimensions[1]/2)), Vec(0,0,0), 2.8, 4)
    end

    local c = proj.effects.color
    PointLight(TransformToParentPoint(proj.transform, Vec(0,0,proj.effects.sprite_dimensions[1]/2)), c[1], c[2], c[3], 50)
	
	--+ Sounds
	PlayLoop(proj.effects.propelSound, proj.transform.pos, 20)
end