function initRadar()
	baseMotor = FindJoint("radarBase")
	mainMotor = FindJoint("radarMain")
	baseMin, baseMax = GetJointLimits(baseMotor)
	
	radarScope = FindLight("radarScope")
	turret = FindShape("turret")
	radarStat = {
		enabled = false,
		raised = false,
		folded = true,
		aligned = true,	--judge whether radar is in right position when folding
		broken = false,
		worldTr = nil,
		localTr = nil,
		turretTr = nil,
		baseAngle = 0,
		mainAngle = 0,
		detectRadius = 1000,
		detectCast = 20,
		target = {},
		radarPos = nil,
		speed = 0,
		const = {
			BASE_SPD = 1,
			SPIN_SPD = 7,
			STOP_SPD = 1,
			SPIN_ACC = 0.05,
			MOTOR_ANG_THR = 0.1,	--deg angle, to judge whether motor has reached position
		},
	}
end

function enableRadar()
	if not radarStat.raised then
		SetJointMotorTarget(baseMotor, baseMax, radarStat.const.BASE_SPD)
	else
		if radarStat.speed < radarStat.const.SPIN_SPD then
			radarStat.speed = radarStat.speed + radarStat.const.SPIN_ACC
		else
			radarStat.speed = radarStat.const.SPIN_SPD
		end
		SetJointMotor(mainMotor, radarStat.speed)
	end
end

function disableRadar()
	if not radarStat.aligned then
		if radarStat.speed > radarStat.const.STOP_SPD then
			radarStat.speed = radarStat.speed - radarStat.const.SPIN_ACC
			SetJointMotor(mainMotor, radarStat.speed)
		else
			SetJointMotorTarget(mainMotor, 0, radarStat.const.STOP_SPD)
		end
	else
		if not radarStat.folded then
			SetJointMotorTarget(baseMotor, baseMin, radarStat.const.BASE_SPD)
		end
	end
end

function updateRadar()
	--atitude
	radarStat.baseAngle = GetJointMovement(baseMotor)
	radarStat.mainAngle = - GetJointMovement(mainMotor)	--counterclockwise as positive
	if radarStat.mainAngle < 0 then
		radarStat.mainAngle = radarStat.mainAngle + 360
	end
	
	if math.abs(radarStat.mainAngle) < 0 + radarStat.const.MOTOR_ANG_THR and radarStat.speed <= radarStat.const.STOP_SPD then
		radarStat.aligned = true
	else
		radarStat.aligned = false
	end
	
	if radarStat.baseAngle < baseMin + radarStat.const.MOTOR_ANG_THR then
		radarStat.folded = true
	else
		radarStat.folded = false
	end
	
	if radarStat.baseAngle > baseMax - radarStat.const.MOTOR_ANG_THR then
		radarStat.raised = true
	else
		radarStat.raised = false
	end
	
	--position
	radarStat.worldTr = GetLightTransform(radarScope)
	--radarStat.radarTr = Transform(GetShapeWorldTransform(radarMount).pos, QuatTrLookDown(GetShapeWorldTransform(baseTurret)))	--make transform face forwards locally
	
	radarStat.turretFwdTr = AabbGetShapeCenteredYawWorldTr(turret)
end

function updateTarget()
	worldVehicles = FindVehicles("",true)
	
	--update target position only if it is within the sight of radar
	for index, handle in pairs(worldVehicles) do
		local pos 			= AabbGetBodyCenterPos(GetVehicleBody(handle))
		local health 		= GetVehicleHealth(handle)
		local tr 			= GetVehicleTransform(handle)
		local dist 			= VecDist(pos, radarStat.worldTr.pos)
		local alt 			= pos[2]
		
		local p1 = TransformToParentPoint(radarStat.turretFwdTr, Vec(0,0,-1))
		local p2 = pos
		local org = AabbGetShapeCenterPos(turret)
		local localAngle 	= VecProjFullAngle(VecSub(p2,org),VecSub(p1,org),"xoz")
		
		
		--update current targets
		if IsPointAffectedByLight(radarScope, pos) then

			local exist = false
			for i=1, #radarStat.target do
				if handle == radarStat.target[i].handle then	--if already exist
					exist = i
					break
				end
			end
			if exist then
				if not radarStat.target[exist].detected then
					radarStat.target[exist].transform = tr
					radarStat.target[exist].pos = pos
					radarStat.target[exist].projDist = dist
					radarStat.target[exist].localAngle = localAngle
					radarStat.target[exist].altitude = alt
					radarStat.target[exist].detected = true
				end
				if not (health > 0) then
					radarStat.target[exist].state = "n"
				end
			else
				if health > 0 then
					local newTg = {
						handle = nil,
						detected = nil,
						locked = nil,
						transform = nil,
						pos = nil,
						projDist = nil,
						localAngle = nil,
						altitude = nil,
						state = nil,
						removeTimer = 0
						}
					newTg.handle = handle
					newTg.transform = tr
					newTg.pos = pos
					newTg.projDist = dist
					newTg.localAngle = localAngle
					newTg.altitude = alt
					newTg.detected = true
					locked = false,
					table.insert(radarStat.target, newTg)
				end
			end
        else
			for i=1, #radarStat.target do
				if radarStat.target[i].handle == handle then
					radarStat.target[i].detected = false
					break
				end
			end
		end
    end
	
	--clear neutralized or missing target
	for i=1, #radarStat.target do
		--missing (dismissed when spawning)
		local dismissed = true
		for index, handle in pairs(worldVehicles) do
			if radarStat.target[i].handle == handle then
				dismissed = false
				break
			end
		end
		if dismissed then
			radarStat.target[i].state = "d"
		end
		
		if radarStat.target[i].state ~= nil then
			radarStat.target[i].removeTimer = radarStat.target[i].removeTimer + GetTimeStep()
			if radarStat.target[i].removeTimer > 3 then
				table.remove(radarStat.target, i)
			end
		end
	end
end